package kerho;

import java.io.File;
import java.util.Collection;
import java.util.List;

/**
 * Kerho-luokka, joka huolehtii jäsenistöstä.  Pääosin kaikki metodit
 * ovat vain "välittäjämetodeja" jäsenistöön.
 *
 * @author Vesa Lappalainen
 * @version 1.0, 09.02.2003
 * @version 1.1, 23.02.2003
 * @version 1.2, 07.01.2008 / testit
 * @version 1.3, 03.03.2013 / Harrastukset
 */
public class Kerho  {
    private Jasenet jasenet = new Jasenet();
    private Harrastukset harrastukset = new Harrastukset();


    /**
     * Poistaa jäsenistöstä ja harrasteista ne joilla on nro. Kesken.
     * @param nro viitenumero, jonka mukaan poistetaan
     * @return montako jäsentä poistettiin
     */
    public int poista(@SuppressWarnings("unused") int nro) {
        return 0;
    }


    /**
     * Lisää kerhoon uuden jäsenen
     * @param jasen lisättävä jäsen
     * @example
     * <pre name="test">
     * Kerho kerho = new Kerho();
     * Jasen aku1 = new Jasen(), aku2 = new Jasen();
     * kerho.lisaa(aku1); 
     * kerho.lisaa(aku2); 
     * kerho.lisaa(aku1);
     * Collection<Jasen> loytyneet = kerho.etsi("",-1); 
     * Iterator<Jasen> it = loytyneet.iterator();
     * it.next() === aku1;
     * it.next() === aku2;
     * it.next() === aku1;
     * </pre>
     */
    public void lisaa(Jasen jasen) {
        jasenet.lisaa(jasen);
    }

    
    /**
     * Korvaa jäsenen jolla sama id
     * @param jasen millä korvataan
     */
    public void korvaaTaiLisaa(Jasen jasen) {
        jasenet.korvaaTaiLisaa(jasen);
    }
    

    /**
     * Listään uusi harrastus kerhoon
     * @param har lisättävä harrastus 
     */
    public void lisaa(Harrastus har)  {
        harrastukset.lisaa(har);
    }


    /**
     * Korvaa harrastuksen tietorakenteessa.  Ottaa harrastuksen omistukseensa.
     * Etsitään samalla tunnusnumerolla oleva harrastus.  Jos ei löydy,
     * niin lisätään uutena harrastuksena.
     * @param harrastus lisärtävän harrastuksen viite.  Huom tietorakenne muuttuu omistajaksi
     * @throws SailoException jos tietorakenne on jo täynnä
     */
    public void korvaaTaiLisaa(Harrastus harrastus) throws SailoException {
        harrastukset.korvaaTaiLisaa(harrastus);
    }


    /**
     * Palauttaa "taulukossa" hakuehtoon vastaavien jäsenten viitteet 
     * @param hakuehto hakuehto  
     * @param k etsittävän kentän indeksi  
     * @return tietorakenteen löytyneistä jäsenistä 
     */ 
    public Collection<Jasen> etsi(String hakuehto, int k) {
        return jasenet.etsi(hakuehto, k); 
    } 
    
    
    /**
     * Haetaan kaikki jäsen harrastukset
     * @param jasen jäsen jolle harrastuksia haetaan
     * @return tietorakenne jossa viiteet löydetteyihin harrastuksiin
     * @example
     * <pre name="test">
     * #import java.util.*;
     * 
     *  Kerho kerho = new Kerho();
     *  Jasen aku1 = new Jasen(), aku2 = new Jasen(), aku3 = new Jasen();
     *  aku1.rekisteroi(); aku2.rekisteroi(); aku3.rekisteroi();
     *  int id1 = aku1.getTunnusNro();
     *  int id2 = aku2.getTunnusNro();
     *  Harrastus pitsi11 = new Harrastus(id1); kerho.lisaa(pitsi11);
     *  Harrastus pitsi12 = new Harrastus(id1); kerho.lisaa(pitsi12);
     *  Harrastus pitsi21 = new Harrastus(id2); kerho.lisaa(pitsi21);
     *  Harrastus pitsi22 = new Harrastus(id2); kerho.lisaa(pitsi22);
     *  Harrastus pitsi23 = new Harrastus(id2); kerho.lisaa(pitsi23);
     *  
     *  List<Harrastus> loytyneet;
     *  loytyneet = kerho.annaHarrastukset(aku3);
     *  loytyneet.size() === 0; 
     *  loytyneet = kerho.annaHarrastukset(aku1);
     *  loytyneet.size() === 2; 
     *  loytyneet.get(0) == pitsi11 === true;
     *  loytyneet.get(1) == pitsi12 === true;
     *  loytyneet = kerho.annaHarrastukset(aku2);
     *  loytyneet.size() === 3; 
     *  loytyneet.get(0) == pitsi21 === true;
     * </pre> 
     */
    public List<Harrastus> annaHarrastukset(Jasen jasen) {
        return harrastukset.annaHarrastukset(jasen.getTunnusNro());
    }


    /**
     * Asettaa tiedostojen perusnimet
     * @param nimi uusi nimi
     */
    public void setTiedosto(String nimi) {
        File dir = new File(nimi);
        dir.mkdirs();
        String hakemistonNimi = "";
        if ( !nimi.isEmpty() ) hakemistonNimi = nimi +"/";
        jasenet.setTiedostonPerusNimi(hakemistonNimi + "nimet");
        harrastukset.setTiedostonPerusNimi(hakemistonNimi + "harrastukset");
    }
    
    
    
    /**
     * Lukee kerhon tiedot tiedostosta
     * @param nimi jota käyteään lukemisessa
     * @throws SailoException jos lukeminen epäonnistuu
     * 
     * @example
     * <pre name="test">
     * #THROWS SailoException 
     * #import java.io.*;
     * #import java.util.*;
     * 
     *  Kerho kerho = new Kerho();
     *  
     *  Jasen aku1 = new Jasen(); aku1.vastaaAkuAnkka(); aku1.rekisteroi();
     *  Jasen aku2 = new Jasen(); aku2.vastaaAkuAnkka(); aku2.rekisteroi();
     *  Harrastus pitsi21 = new Harrastus(); pitsi21.vastaaPitsinNyplays(aku2.getTunnusNro());
     *  Harrastus pitsi11 = new Harrastus(); pitsi11.vastaaPitsinNyplays(aku1.getTunnusNro());
     *  Harrastus pitsi22 = new Harrastus(); pitsi22.vastaaPitsinNyplays(aku2.getTunnusNro()); 
     *  Harrastus pitsi12 = new Harrastus(); pitsi12.vastaaPitsinNyplays(aku1.getTunnusNro()); 
     *  Harrastus pitsi23 = new Harrastus(); pitsi23.vastaaPitsinNyplays(aku2.getTunnusNro());
     *   
     *  String hakemisto = "testikelmit";
     *  File dir = new File(hakemisto);
     *  File ftied  = new File(hakemisto+"/nimet.dat");
     *  File fhtied = new File(hakemisto+"/harrastukset.dat");
     *  dir.mkdir();  
     *  ftied.delete();
     *  fhtied.delete();
     *  kerho.lueTiedostosta(hakemisto); #THROWS SailoException
     *  kerho.lisaa(aku1);
     *  kerho.lisaa(aku2);
     *  kerho.lisaa(pitsi21);
     *  kerho.lisaa(pitsi11);
     *  kerho.lisaa(pitsi22);
     *  kerho.lisaa(pitsi12);
     *  kerho.lisaa(pitsi23);
     *  kerho.tallenna();
     *  kerho = new Kerho();
     *  kerho.lueTiedostosta(hakemisto);
     *  Collection<Jasen> kaikki = kerho.etsi("",-1); 
     *  Iterator<Jasen> it = kaikki.iterator();
     *  it.next() === aku1;
     *  it.next() === aku2;
     *  it.hasNext() === false;
     *  List<Harrastus> loytyneet = kerho.annaHarrastukset(aku1);
     *  Iterator<Harrastus> ih = loytyneet.iterator();
     *  ih.next() === pitsi11;
     *  ih.next() === pitsi12;
     *  ih.hasNext() === false;
     *  loytyneet = kerho.annaHarrastukset(aku2);
     *  ih = loytyneet.iterator();
     *  ih.next() === pitsi21;
     *  ih.next() === pitsi22;
     *  ih.next() === pitsi23;
     *  ih.hasNext() === false;
     *  kerho.lisaa(aku2);
     *  kerho.lisaa(pitsi23);
     *  kerho.tallenna();
     *  ftied.delete()  === true;
     *  fhtied.delete() === true;
     *  File fbak = new File(hakemisto+"/nimet.bak");
     *  File fhbak = new File(hakemisto+"/harrastukset.bak");
     *  fbak.delete() === true;
     *  fhbak.delete() === true;
     *  dir.delete() === true;
     * </pre>
     */
    public void lueTiedostosta(String nimi) throws SailoException {
        jasenet = new Jasenet(); // jos luetaan olemassa olevaan niin helpoin tyhjentää näin
        harrastukset = new Harrastukset();

        setTiedosto(nimi);
        jasenet.lueTiedostosta();
        harrastukset.lueTiedostosta();
    }


    /**
     * Tallenttaa kerhon tiedot tiedostoon.  
     * Vaikka jäsenten tallettamien epäonistuisi, niin yritetään silti tallettaa
     * harrastuksia ennen poikkeuksen heittämistä.
     * @throws SailoException jos tallettamisessa ongelmia
     */
    public void tallenna() throws SailoException {
        String virhe = "";
        try {
            jasenet.tallenna();
        } catch ( SailoException ex ) {
            virhe = ex.getMessage();
        }

        try {
            harrastukset.tallenna();
        } catch ( SailoException ex ) {
            virhe += ex.getMessage();
        }
        if ( !"".equals(virhe) ) throw new SailoException(virhe);
    }


    /**
     * Testiohjelma kerhosta
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Kerho kerho = new Kerho();

        // kerho.lueTiedostosta("kelmit");

        Jasen aku1 = new Jasen(), aku2 = new Jasen();
        aku1.rekisteroi();
        aku1.vastaaAkuAnkka();
        aku2.rekisteroi();
        aku2.vastaaAkuAnkka();

        kerho.lisaa(aku1);
        kerho.lisaa(aku2);
        int id1 = aku1.getTunnusNro();
        int id2 = aku2.getTunnusNro();
        Harrastus pitsi11 = new Harrastus(id1);
        pitsi11.vastaaPitsinNyplays(id1);
        kerho.lisaa(pitsi11);
        Harrastus pitsi12 = new Harrastus(id1);
        pitsi12.vastaaPitsinNyplays(id1);
        kerho.lisaa(pitsi12);
        Harrastus pitsi21 = new Harrastus(id2);
        pitsi21.vastaaPitsinNyplays(id2);
        kerho.lisaa(pitsi21);
        Harrastus pitsi22 = new Harrastus(id2);
        pitsi22.vastaaPitsinNyplays(id2);
        kerho.lisaa(pitsi22);
        Harrastus pitsi23 = new Harrastus(id2);
        pitsi23.vastaaPitsinNyplays(id2);
        kerho.lisaa(pitsi23);

        System.out.println("============= Kerhon testi =================");
        Collection<Jasen> jasenet = kerho.etsi("", -1);
        int i = 0;
        for (Jasen jasen: jasenet) {
            System.out.println("Jäsen paikassa: " + i);
            jasen.tulosta(System.out);
            List<Harrastus> loytyneet = kerho.annaHarrastukset(jasen);
            for (Harrastus harrastus : loytyneet)
                harrastus.tulosta(System.out);
            i++;
        }

    }

}
